@@ -1,18 +1,16 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2012 CORE Security Technologies
+# Copyright (c) 2003-2015 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: atexec.py 876 2013-10-29 16:04:19Z bethus@gmail.com $
-#
 # ATSVC example for some functions implemented, creates, enums, runs, delete jobs
 # This example executes a command on the target machine through the Task Scheduler 
 # service. Returns the output of such command
 #
 # Author:
-#  Alberto Solino (bethus@gmail.com)
+#  Alberto Solino (@agsolino)
 #
 # Reference for:
 #  DCE/RPC for ATSVC
@@ -24,9 +22,11 @@ import types
 import argparse
 import time
 import random
+import logging
 
 from impacket import uuid, ntlm, version
 from impacket.dcerpc import transport, ndrutils, atsvc
+from impacket.examples import logger
 from struct import unpack
 
 class ATSVC_EXEC:
@@ -55,7 +55,7 @@ class ATSVC_EXEC:
             protodef = ATSVC_EXEC.KNOWN_PROTOCOLS[protocol]
             port = protodef[1]
 
-            print "Trying protocol %s..." % protocol
+            logging.info("Trying protocol %s..." % protocol)
             stringbinding = protodef[0] % addr
 
             rpctransport = transport.DCERPCTransportFactory(stringbinding)
@@ -66,7 +66,7 @@ class ATSVC_EXEC:
             try:
                 self.doStuff(rpctransport)
             except Exception, e:
-                print 'Protocol failed: %s' % e
+                logging.error(e)
             else:
                 # Got a response. No need for further iterations.
                 break
@@ -152,11 +152,11 @@ class ATSVC_EXEC:
 # Process command-line arguments.
 if __name__ == '__main__':
     print version.BANNER
-    print "WARNING: This will work ONLY on Windows >= Vista"
+    logging.warning("This will work ONLY on Windows >= Vista")
 
     parser = argparse.ArgumentParser()
 
-    parser.add_argument('target', action='store', help='[domain/][username[:password]@]<address>')
+    parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
     parser.add_argument('command', action='store', nargs='*', default = ' ', help='command to execute at the target ')
 
     group = parser.add_argument_group('authentication')
