@@ -1,17 +1,15 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2013 CORE Security Technologies
+# Copyright (c) 2003-2015 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: esentutl.py 1225 2014-05-23 18:51:05Z bethus@gmail.com $
-#
 # Description:
 #             ESE utility. Allows dumping catalog, pages and tables.
 #
 # Author:
-#  Alberto Solino
+#  Alberto Solino (@agsolino)
 #
 #
 # Reference for:
@@ -23,6 +21,7 @@ import logging
 import argparse
 from impacket import version, ese
 from impacket.ese import ESENT_DB
+from impacket.examples import logger
 
 def dumpPage(ese, pageNum):
     data = ese.getPage(pageNum)
@@ -53,7 +52,7 @@ def exportTable(ese, tableName):
 
 def main():
     print version.BANNER
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(add_help = True, description = "Extensive Storage Engine utility. Allows dumping catalog, pages and tables.")
     parser.add_argument('databaseFile', action='store', help='ESE to open')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
     parser.add_argument('-page', action='store', help='page to open')
