@@ -1,27 +1,27 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2012 CORE Security Technologies
+# Copyright (c) 2003-2015 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: mssqlclient.py 833 2013-10-17 20:21:17Z bethus@gmail.com $
-#
 # Description: [MS-TDS] & [MC-SQLR] example.
 #
 # Author:
-#  Alberto Solino (beto@coresecurity.com)
+#  Alberto Solino (beto@coresecurity.com/@agsolino)
 #
 # Reference for:
 #  Structure
 #
 
 
-from impacket import version, tds
 import argparse
 import sys
 import string
 import os
+import logging
+from impacket import version, tds
+from impacket.examples import logger
 
 if __name__ == '__main__':
     import cmd
@@ -93,12 +93,13 @@ if __name__ == '__main__':
 
     print version.BANNER
 
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(add_help = True, description = "TDS client implementation (SSL supported).")
 
-    parser.add_argument('target', action='store', help='[domain/][username[:password]@]<address>')
+    parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
     parser.add_argument('-port', action='store', default='1433', help='target MSSQL port (default 1433)')
     parser.add_argument('-db', action='store', help='MSSQL database instance (default None)')
     parser.add_argument('-windows-auth', action='store_true', default = 'False', help='whether or not to use Windows Authentication (default False)')
+    parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
     parser.add_argument('-file', type=argparse.FileType('r'), help='input file with commands to execute in the SQL shell')
 
     group = parser.add_argument_group('authentication')
@@ -110,6 +111,11 @@ if __name__ == '__main__':
  
     options = parser.parse_args()
 
+    if options.debug is True:
+        logging.getLogger().setLevel(logging.DEBUG)
+    else:
+        logging.getLogger().setLevel(logging.INFO)
+
     import re
     domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
 
