@@ -1,16 +1,14 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2012 CORE Security Technologies
+# Copyright (c) 2003-2015 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: mssqlinstance.py 631 2012-07-24 16:43:04Z bethus@gmail.com $
-#
 # Description: [MC-SQLR] example. Retrieves the instances names from the target host
 #
 # Author:
-#  Alberto Solino (beto@coresecurity.com)
+#  Alberto Solino (@agsolino)
 #
 # Reference for:
 #  Structure
@@ -18,6 +16,7 @@
 
 
 from impacket import version, tds
+from impacket.examples import logger
 import argparse
 import sys
 import string
@@ -27,13 +26,12 @@ if __name__ == '__main__':
 
     print version.BANNER
 
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(add_help = True, description = "Asks the remote host for its running MSSQL Instances.")
 
     parser.add_argument('host', action='store', help='target host')
     parser.add_argument('-timeout', action='store', default='5', help='timeout to wait for an answer')
 
     if len(sys.argv)==1:
-        print "description: asks the remote host for its running MSSQL Instances\n"
         parser.print_help()
         sys.exit(1)
  
@@ -42,10 +40,10 @@ if __name__ == '__main__':
     ms_sql = tds.MSSQL(options.host)
     instances = ms_sql.getInstances(string.atoi(options.timeout))
     if len(instances) == 0:
-        print "No MSSQL Instances found"
+        "No MSSQL Instances found"
     else:
         for i, instance in enumerate(instances):
-            print "[*] Instance %d" % i
+            logging.info("Instance %d" % i)
             for key in instance.keys():
                print key + ":" + instance[key]
  
