@@ -1,16 +1,14 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2013 CORE Security Technologies
+# Copyright (c) 2003-2015 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: ntfs-read.py 1113 2014-01-21 20:58:34Z bethus@gmail.com $
-#
 # Description: Mini shell for browsing an NTFS volume
 #
 # Author:
-#  Alberto Solino
+#  Alberto Solino (@agsolino)
 #
 #
 # Reference for:
@@ -41,7 +39,7 @@ except ImportError:
 from datetime import datetime
 from impacket import structure, version
 from impacket.structure import Structure
-from impacket.dcerpc.samr import SAMR_RPC_SID_IDENTIFIER_AUTHORITY, SAMR_RPC_SID
+from impacket.examples import logger
 
 import string
 def pretty_print(x):
@@ -1035,7 +1033,7 @@ class MiniShell(cmd.Cmd):
         try:
            retVal = cmd.Cmd.onecmd(self,s)
         except Exception, e:
-           print "ERROR: %s" % e
+           logging.error(str(e))
 
         return retVal
 
@@ -1082,11 +1080,11 @@ class MiniShell(cmd.Cmd):
             res = self.findPathName(newPath)
 
         if res is None:
-            print "Directory not found"
+            logging.error("Directory not found")
             self.pwd = oldpwd
             return 
         if res.isDirectory() == 0:
-            print "Not a directory!"
+            logging.error("Not a directory!")
             self.pwd = oldpwd
             return
         else:
@@ -1166,10 +1164,10 @@ class MiniShell(cmd.Cmd):
         pathName = ntpath.normpath(ntpath.join(self.pwd,pathName))
         res = self.findPathName(pathName)
         if res is None:
-            print "Not found!"
+            logging.error("Not found!")
             return
         if res.isDirectory() > 0:
-            print "It's a directory!"
+            logging.error("It's a directory!")
             return
         if res.isCompressed() or res.isEncrypted() or res.isSparse():
             logging.error('Cannot handle compressed/encrypted/sparse files! :(')
@@ -1184,7 +1182,7 @@ class MiniShell(cmd.Cmd):
         if stream.getDataSize() % chunks:
             buf = stream.read(written, stream.getDataSize() % chunks)
             command(buf)
-        print "%d bytes read" % stream.getDataSize()
+        logging.info("%d bytes read" % stream.getDataSize())
 
     def do_get(self, line):
         pathName = string.replace(line,'/','\\')
@@ -1196,13 +1194,21 @@ class MiniShell(cmd.Cmd):
 def main():
     print version.BANNER
     logging.getLogger().setLevel(logging.WARNING)
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(add_help = True, description = "NTFS explorer (read-only)")
     parser.add_argument('volume', action='store', help='NTFS volume to open (e.g. \\\\.\\C: or /dev/disk1s1)')
     parser.add_argument('-extract', action='store', help='extracts pathname (e.g. \windows\system32\config\sam)')
+    parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
+
     if len(sys.argv)==1:
         parser.print_help()
         sys.exit(1)
     options = parser.parse_args()
+
+    if options.debug is True:
+        logging.getLogger().setLevel(logging.DEBUG)
+    else:
+        logging.getLogger().setLevel(logging.INFO)
+
     shell = MiniShell(options.volume)
     if options.extract is not None:
         shell.onecmd("get %s"% options.extract)
