@@ -25,7 +25,8 @@ Author: Catalin Patulea <cat@vv.carleton.ca>
 """
 import sys
 from impacket import uuid
-from impacket.dcerpc import transport, dcerpc
+from impacket.dcerpc.v5 import transport
+from impacket.examples import logging
 
 def main(args):
   if len(args) != 4:
@@ -49,7 +50,7 @@ def main(args):
     dce.call(i, "")
     try:
       resp = dce.recv()
-    except dcerpc.Exception, e:
+    except Exception, e:
       result = str(e)
     else:
       result = "success"
