@@ -1,16 +1,14 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2012 CORE Security Technologies
+# Copyright (c) 2003-2015 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: psexec.py 876 2013-10-29 16:04:19Z bethus@gmail.com $
-#
 # PSEXEC like functionality example using RemComSvc (https://github.com/kavika13/RemCom)
 #
 # Author:
-#  beto (bethus@gmail.com)
+#  beto (@agsolino)
 #
 # Reference for:
 #  DCE/RPC and SMB.
@@ -18,13 +16,14 @@
 import sys
 import os
 import cmd
+import logging
 
 from impacket import version
 from impacket.smbconnection import *
-from impacket.dcerpc import transport, svcctl
+from impacket.dcerpc.v5 import transport
 from impacket.structure import Structure
 from threading import Thread, Lock
-from impacket.examples import remcomsvc, serviceinstall
+from impacket.examples import remcomsvc, serviceinstall, logger
 import argparse
 import random
 import string
@@ -58,9 +57,8 @@ class PSEXEC:
         '445/SMB': (r'ncacn_np:%s[\pipe\svcctl]', 445),
         }
 
-
     def __init__(self, command, path, exeFile, copyFile, protocols = None, 
-                 username = '', password = '', domain = '', hashes = None):
+                 username = '', password = '', domain = '', hashes = None, aesKey = None, doKerberos = False):
         self.__username = username
         self.__password = password
         if protocols is None:
@@ -72,8 +70,10 @@ class PSEXEC:
         self.__domain = domain
         self.__lmhash = ''
         self.__nthash = ''
+        self.__aesKey = aesKey
         self.__exeFile = exeFile
         self.__copyFile = copyFile
+        self.__doKerberos = doKerberos
         if hashes is not None:
             self.__lmhash, self.__nthash = hashes.split(':')
 
@@ -82,7 +82,7 @@ class PSEXEC:
             protodef = PSEXEC.KNOWN_PROTOCOLS[protocol]
             port = protodef[1]
 
-            print "Trying protocol %s...\n" % protocol
+            logging.info("Trying protocol %s...\n" % protocol)
             stringbinding = protodef[0] % addr
 
             rpctransport = transport.DCERPCTransportFactory(stringbinding)
@@ -91,8 +91,9 @@ class PSEXEC:
             #   rpctransport.preferred_dialect(SMB_DIALECT)
             if hasattr(rpctransport, 'set_credentials'):
                 # This method exists only for selected protocol sequences.
-                rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash)
+                rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash, self.__aesKey)
 
+            rpctransport.set_kerberos(self.__doKerberos)
             self.doStuff(rpctransport)
 
     def openPipe(self, s, tid, pipe, accessMask):
@@ -108,7 +109,7 @@ class PSEXEC:
                 pass
 
         if tries == 0:
-            print '[!] Pipe not ready, aborting'
+            logging.critical('Pipe not ready, aborting')
             raise
 
         fid = s.openFile(tid,pipe,accessMask, creationOption = 0x40, fileAttributes = 0x80)
@@ -121,7 +122,7 @@ class PSEXEC:
         try:
             dce.connect()
         except Exception, e:
-            print e
+            logging.critical(str(e))
             sys.exit(1)
 
         global dialect
@@ -139,7 +140,7 @@ class PSEXEC:
                 try:
                     f = open(self.__exeFile)
                 except Exception, e:
-                    print e
+                    logging.critical(str(e))
                     sys.exit(1)
                 installService = serviceinstall.ServiceInstall(rpctransport.get_smb_connection(), f)
     
@@ -186,7 +187,7 @@ class PSEXEC:
 
             if len(ans):
                retCode = RemComResponse(ans)
-               print "[*] Process %s finished with ErrorCode: %d, ReturnCode: %d" % (self.__command, retCode['ErrorCode'], retCode['ReturnCode'])
+               logging.info("Process %s finished with ErrorCode: %d, ReturnCode: %d" % (self.__command, retCode['ErrorCode'], retCode['ReturnCode']))
             installService.uninstall()
             if self.__copyFile is not None:
                 # We copied a file for execution, let's remove it
@@ -204,8 +205,6 @@ class PSEXEC:
             sys.stdout.flush()
             sys.exit(1)
 
-
-
 class Pipes(Thread):
     def __init__(self, transport, pipe, permissions, share=None):
         Thread.__init__(self)
@@ -226,8 +225,11 @@ class Pipes(Thread):
             global dialect
             #self.server = SMBConnection('*SMBSERVER', self.transport.get_smb_connection().getRemoteHost(), sess_port = self.port, preferredDialect = SMB_DIALECT)
             self.server = SMBConnection('*SMBSERVER', self.transport.get_smb_connection().getRemoteHost(), sess_port = self.port, preferredDialect = dialect)
-            user, passwd, domain, lm, nt = self.credentials
-            self.server.login(user, passwd, domain, lm, nt)
+            user, passwd, domain, lm, nt, aesKey, TGT, TGS = self.credentials
+            if self.transport.get_kerberos() is True:
+                self.server.kerberosLogin(user, passwd, domain, lm, nt, aesKey, TGT=TGT, TGS=TGS)
+            else:
+                self.server.login(user, passwd, domain, lm, nt)
             lock.release()
             self.tid = self.server.connectTree('IPC$') 
 
@@ -235,7 +237,7 @@ class Pipes(Thread):
             self.fid = self.server.openFile(self.tid,self.pipe,self.permissions, creationOption = 0x40, fileAttributes = 0x80)
             self.server.setTimeout(1000000)
         except:
-            print "[!] Something wen't wrong connecting the pipes(%s), try again" % self.__class__
+            logging.error("Something wen't wrong connecting the pipes(%s), try again" % self.__class__)
 
 
 class RemoteStdOutPipe(Pipes):
@@ -251,17 +253,17 @@ class RemoteStdOutPipe(Pipes):
                 pass
             else:
                 try:
-                        global LastDataSent
-                        if ans != LastDataSent:
-                            sys.stdout.write(ans)
-                            sys.stdout.flush()
-                        else:
-                            # Don't echo what I sent, and clear it up
-                            LastDataSent = ''
-                        # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars, 
-                        # it will give false positives tho.. we should find a better way to handle this.
-                        if LastDataSent > 10:
-                            LastDataSent = ''
+                    global LastDataSent
+                    if ans != LastDataSent:
+                        sys.stdout.write(ans)
+                        sys.stdout.flush()
+                    else:
+                        # Don't echo what I sent, and clear it up
+                        LastDataSent = ''
+                    # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars, 
+                    # it will give false positives tho.. we should find a better way to handle this.
+                    if LastDataSent > 10:
+                        LastDataSent = ''
                 except:
                     pass
 
@@ -284,7 +286,7 @@ class RemoteStdErrPipe(Pipes):
                     pass
 
 class RemoteShell(cmd.Cmd):
-    def __init__(self, server, port, credentials, tid, fid, share):
+    def __init__(self, server, port, credentials, tid, fid, share, transport):
         cmd.Cmd.__init__(self, False)
         self.prompt = '\x08'
         self.server = server
@@ -294,13 +296,17 @@ class RemoteShell(cmd.Cmd):
         self.credentials = credentials
         self.share = share
         self.port = port
+        self.transport = transport
         self.intro = '[!] Press help for extra shell commands'
 
     def connect_transferClient(self):
         #self.transferClient = SMBConnection('*SMBSERVER', self.server.getRemoteHost(), sess_port = self.port, preferredDialect = SMB_DIALECT)
         self.transferClient = SMBConnection('*SMBSERVER', self.server.getRemoteHost(), sess_port = self.port, preferredDialect = dialect)
-        user, passwd, domain, lm, nt = self.credentials
-        self.transferClient.login(user, passwd, domain, lm, nt)
+        user, passwd, domain, lm, nt, aesKey, TGT, TGS = self.credentials
+        if self.transport.get_kerberos() is True:
+            self.transferClient.kerberosLogin(user, passwd, domain, lm, nt, aesKey, TGT=TGT, TGS=TGS)
+        else:
+            self.transferClient.login(user, passwd, domain, lm, nt)
 
     def do_help(self, line):
         print """
@@ -324,11 +330,11 @@ class RemoteShell(cmd.Cmd):
             import ntpath
             filename = ntpath.basename(src_path)
             fh = open(filename,'wb')
-            print "[*] Downloading %s\%s" % (self.share, src_path)
+            logging.info("Downloading %s\%s" % (self.share, src_path))
             self.transferClient.getFile(self.share, src_path, fh.write)
             fh.close()
         except Exception, e:
-            print e
+            logging.critical(str(e))
             pass
 
         self.send_data('\r\n')
@@ -349,16 +355,15 @@ class RemoteShell(cmd.Cmd):
             fh = open(src_path, 'rb')
             f = dst_path + '/' + src_file
             pathname = string.replace(f,'/','\\')
-            print "[*] Uploading %s to %s\%s" % (src_file, self.share, dst_path)
+            logging.info("Uploading %s to %s\%s" % (src_file, self.share, dst_path))
             self.transferClient.putFile(self.share, pathname, fh.read)
             fh.close()
         except Exception, e:
-            print e
+            logging.error(str(e))
             pass
 
         self.send_data('\r\n')
 
-
     def do_lcd(self, s):
         if s == '':
             print os.getcwd()
@@ -381,48 +386,62 @@ class RemoteShell(cmd.Cmd):
             LastDataSent = ''
         self.server.writeFile(self.tid, self.fid, data)
 
-
 class RemoteStdInPipe(Pipes):
     def __init__(self, transport, pipe, permisssions, share=None):
         Pipes.__init__(self, transport, pipe, permisssions, share)
 
     def run(self):
         self.connectPipe()
-        self.shell = RemoteShell(self.server, self.port, self.credentials, self.tid, self.fid, self.share)
+        self.shell = RemoteShell(self.server, self.port, self.credentials, self.tid, self.fid, self.share, self.transport)
         self.shell.cmdloop()
 
-
 # Process command-line arguments.
 if __name__ == '__main__':
     print version.BANNER
 
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(add_help = True, description = "PSEXEC like functionality example using RemComSvc.")
 
-    parser.add_argument('target', action='store', help='[domain/][username[:password]@]<address>')
-    parser.add_argument('command', nargs='*', default = ' ', help='command (or arguments if -c is used) to execute at the target (w/o path)')
+    parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
+    parser.add_argument('command', nargs='*', default = ' ', help='command (or arguments if -c is used) to execute at the target (w/o path) - (default:cmd.exe)')
     parser.add_argument('-c', action='store',metavar = "pathname",  help='copy the filename for later execution, arguments are passed in the command option')
     parser.add_argument('-path', action='store', help='path of the command to execute')
     parser.add_argument('-file', action='store', help="alternative RemCom binary (be sure it doesn't require CRT)")
+    parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
 
     group = parser.add_argument_group('authentication')
 
     group.add_argument('-hashes', action="store", metavar = "LMHASH:NTHASH", help='NTLM hashes, format is LMHASH:NTHASH')
- 
+    group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
+    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file (KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the ones specified in the command line')
+    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication (128 or 256 bits)')
+
     if len(sys.argv)==1:
         parser.print_help()
         sys.exit(1)
 
     options = parser.parse_args()
 
+    if options.debug is True:
+        logging.getLogger().setLevel(logging.DEBUG)
+    else:
+        logging.getLogger().setLevel(logging.INFO)
+
     import re
     domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
 
     if domain is None:
         domain = ''
 
-    if password == '' and username != '' and options.hashes is None:
+    if password == '' and username != '' and options.hashes is None and options.no_pass is False and options.aesKey is None:
         from getpass import getpass
         password = getpass("Password:")
 
-    executer = PSEXEC(' '.join(options.command), options.path, options.file, options.c, None, username, password, domain, options.hashes)
+    if options.aesKey is not None:
+        options.k = True
+
+    command = ' '.join(options.command)
+    if command == ' ':
+        command = 'cmd.exe'
+
+    executer = PSEXEC(command, options.path, options.file, options.c, None, username, password, domain, options.hashes, options.aesKey, options.k)
     executer.run(address)
