@@ -1,14 +1,12 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2013 CORE Security Technologies
+# Copyright (c) 2003-2015 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: rdp_check.py 904 2013-11-07 21:21:13Z bethus@gmail.com $
-#
 # Author:
-#  Alberto Solino (beto@coresecurity.com or bethus@gmail.com)
+#  Alberto Solino (@agsolino)
 #
 # Description: [MS-RDPBCGR] and [MS-CREDSSP] partial implementation 
 #              just to reach CredSSP auth. This example test whether
@@ -22,6 +20,7 @@
 from impacket.structure import Structure
 from impacket.spnego import *
 from struct import pack, unpack
+from impacket.examples import logger
 
 TDPU_CONNECTION_REQUEST  = 0xe0
 TPDU_CONNECTION_CONFIRM  = 0xd0
@@ -284,7 +283,7 @@ if __name__ == '__main__':
         import OpenSSL
         from OpenSSL import SSL, crypto
     except:
-        print "pyOpenSSL is not installed, can't continue"
+        logging.critical("pyOpenSSL is not installed, can't continue")
         sys.exit(1)
     
 
@@ -488,7 +487,7 @@ if __name__ == '__main__':
            buff = tls.recv(1024)
        except Exception, err:
            if str(err).find("denied") > 0:
-               print "[*] Access Denied"
+               logging.error("Access Denied")
            else:
                logging.error(err)
            return
@@ -540,13 +539,13 @@ if __name__ == '__main__':
        ts_request['authInfo'] = str(signature) + cripted_creds
        tls.send(ts_request.getData())
        tls.close()
-       print "[*] Access Granted"
+       logging.info("Access Granted")
 
     print version.BANNER
 
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(add_help = True, description = "Test whether an account is valid on the target host using the RDP protocol.")
 
-    parser.add_argument('target', action='store', help='[domain/][username[:password]@]<address>')
+    parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
 
     group = parser.add_argument_group('authentication')
 
