@@ -5,8 +5,6 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: rpcdump.py 1076 2014-01-09 19:17:14Z bethus@gmail.com $
-#
 # DCE/RPC endpoint mapper dumper.
 #
 # Author:
@@ -20,10 +18,12 @@ import socket
 import string
 import sys
 import types
+import logging
 
 from impacket import uuid, ntlm, version
 from impacket.dcerpc.v5 import transport, epm
 from impacket.dcerpc import  ndrutils
+from impacket.examples import logger
 import argparse
 
 class RPCDump:
@@ -54,7 +54,7 @@ class RPCDump:
         string format.
         """
 
-        print 'Retrieving endpoint list from %s' % addr
+        logging.info('Retrieving endpoint list from %s' % addr)
 
         # Try all requested protocols until one works.
         entries = []
@@ -62,7 +62,7 @@ class RPCDump:
             protodef = RPCDump.KNOWN_PROTOCOLS[protocol]
             port = protodef[1]
 
-            print "Trying protocol %s..." % protocol
+            logging.info("Trying protocol %s..." % protocol)
             stringbinding = protodef[0] % addr
 
             rpctransport = transport.DCERPCTransportFactory(stringbinding)
@@ -74,7 +74,7 @@ class RPCDump:
             try:
                 entries = self.__fetchList(rpctransport)
             except Exception, e:
-                print 'Protocol failed: %s' % e
+                logging.critical('Protocol failed: %s' % e)
             else:
                 # Got a response. No need for further iterations.
                 break
@@ -115,11 +115,11 @@ class RPCDump:
         if entries:
             num = len(entries)
             if 1 == num:
-                print 'Received one endpoint.'
+                logging.info('Received one endpoint.')
             else:
-                print 'Received %d endpoints.' % num
+                logging.info('Received %d endpoints.' % num)
         else:
-            print 'No endpoints found.'
+            logging.info('No endpoints found.')
 
 
     def __fetchList(self, rpctransport):
@@ -142,9 +142,9 @@ class RPCDump:
 if __name__ == '__main__':
     print version.BANNER
 
-    parser = argparse.ArgumentParser()
-
-    parser.add_argument('target', action='store', help='[domain/][username[:password]@]<address>')
+    parser = argparse.ArgumentParser(add_help = True, description = "Dumps the remote RPC enpoints information.")
+    parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
+    parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
     parser.add_argument('protocol', choices=RPCDump.KNOWN_PROTOCOLS.keys(), nargs='?', default='135/TCP', help='transport protocol (default 135/TCP)')
 
     group = parser.add_argument_group('authentication')
@@ -156,6 +156,11 @@ if __name__ == '__main__':
  
     options = parser.parse_args()
 
+    if options.debug is True:
+        logging.getLogger().setLevel(logging.DEBUG)
+    else:
+        logging.getLogger().setLevel(logging.INFO)
+
     import re
     domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
 
