@@ -1,12 +1,10 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2014 CORE Security Technologies
+# Copyright (c) 2003-2015 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: samrdump.py 1077 2014-01-09 23:25:44Z bethus@gmail.com $
-#
 # Description: DCE/RPC SAMR dumper.
 #
 # Author:
@@ -20,10 +18,12 @@ import socket
 import string
 import sys
 import types
+import logging
 
 from impacket import uuid, version
 from impacket.nt_errors import STATUS_MORE_ENTRIES
 from impacket.dcerpc.v5 import transport, samr
+from impacket.examples import logger
 import argparse
 
 
@@ -38,7 +38,7 @@ class SAMRDump:
 
 
     def __init__(self, protocols = None,
-                 username = '', password = '', domain = '', hashes = None):
+                 username = '', password = '', domain = '', hashes = None, aesKey=None, doKerberos = False):
         if not protocols:
             self.__protocols = SAMRDump.KNOWN_PROTOCOLS.keys()
         else:
@@ -49,6 +49,8 @@ class SAMRDump:
         self.__domain = domain
         self.__lmhash = ''
         self.__nthash = ''
+        self.__aesKey = aesKey
+        self.__doKerberos = doKerberos
         if hashes is not None:
             self.__lmhash, self.__nthash = hashes.split(':')
 
@@ -58,7 +60,7 @@ class SAMRDump:
         addr. Addr is a valid host name or IP address.
         """
 
-        print 'Retrieving endpoint list from %s' % addr
+        logging.info('Retrieving endpoint list from %s' % addr)
 
         # Try all requested protocols until one works.
         entries = []
@@ -66,13 +68,13 @@ class SAMRDump:
             protodef = SAMRDump.KNOWN_PROTOCOLS[protocol]
             port = protodef[1]
 
-            print "Trying protocol %s..." % protocol
-            rpctransport = transport.SMBTransport(addr, port, r'\samr', self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash)
+            logging.info("Trying protocol %s..." % protocol)
+            rpctransport = transport.SMBTransport(addr, port, r'\samr', self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash, self.__aesKey, doKerberos = self.__doKerberos)
 
             try:
                 entries = self.__fetchList(rpctransport)
             except Exception, e:
-                print 'Protocol failed: %s' % e
+                logging.critical(str(e))
             else:
                 # Got a response. No need for further iterations.
                 break
@@ -91,11 +93,11 @@ class SAMRDump:
         if entries:
             num = len(entries)
             if 1 == num:
-                print 'Received one entry.'
+                logging.info('Received one entry.')
             else:
-                print 'Received %d entries.' % num
+                logging.info('Received %d entries.' % num)
         else:
-            print 'No entries received.'
+            logging.info('No entries received.')
 
 
     def __fetchList(self, rpctransport):
@@ -117,7 +119,7 @@ class SAMRDump:
             for domain in domains:
                 print " . %s" % domain['Name']
 
-            print "Looking up users in domain %s" % domains[0]['Name']
+            logging.info("Looking up users in domain %s" % domains[0]['Name'])
 
             resp = samr.hSamrLookupDomainInSamServer(dce, serverHandle,domains[0]['Name'] )
 
@@ -149,7 +151,7 @@ class SAMRDump:
                 status = resp['ErrorCode']
 
         except ListUsersException, e:
-            print "Error listing users: %s" % e
+            logging.critical("Error listing users: %s" % e)
 
         dce.disconnect()
 
@@ -160,20 +162,30 @@ class SAMRDump:
 if __name__ == '__main__':
     print version.BANNER
 
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(add_help = True, description = "This script downloads the list of users for the target system.")
 
-    parser.add_argument('target', action='store', help='[domain/][username[:password]@]<address>')
+    parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
     parser.add_argument('protocol', choices=SAMRDump.KNOWN_PROTOCOLS.keys(), nargs='?', default='445/SMB', help='transport protocol (default 445/SMB)')
+    parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
 
     group = parser.add_argument_group('authentication')
 
     group.add_argument('-hashes', action="store", metavar = "LMHASH:NTHASH", help='NTLM hashes, format is LMHASH:NTHASH')
+    group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
+    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file (KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the ones specified in the command line')
+    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication (128 or 256 bits)')
+
     if len(sys.argv)==1:
         parser.print_help()
         sys.exit(1)
 
     options = parser.parse_args()
 
+    if options.debug is True:
+        logging.getLogger().setLevel(logging.DEBUG)
+    else:
+        logging.getLogger().setLevel(logging.INFO)
+
     import re
 
     domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
@@ -181,9 +193,12 @@ if __name__ == '__main__':
     if domain is None:
         domain = ''
 
-    if password == '' and username != '' and options.hashes is None:
+    if options.aesKey is not None:
+        options.k = True
+
+    if password == '' and username != '' and options.hashes is None and options.no_pass is False and options.aesKey is None:
         from getpass import getpass
         password = getpass("Password:")
 
-    dumper = SAMRDump(options.protocol, username, password, domain, options.hashes)
+    dumper = SAMRDump(options.protocol, username, password, domain, options.hashes, options.aesKey, options.k)
     dumper.dump(address)
