@@ -1,16 +1,14 @@
 #!/usr/bin/python
-# Copyright (c) 2003-2014 CORE Security Technologies
+# Copyright (c) 2003-2015 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: smbclient.py 1125 2014-01-27 19:58:16Z bethus@gmail.com $
-#
 # Description: Mini shell using some of the SMB funcionality of the library
 #
 # Author:
-#  Alberto Solino
+#  Alberto Solino (@agsolino)
 #
 # 
 # Reference for:
@@ -21,7 +19,8 @@ import sys
 import string
 import time
 import logging
-from impacket import smb, version, smb3, nt_errors
+from impacket.examples import logger 
+from impacket import smb, version, smb3, nt_errors, version
 from impacket.dcerpc.v5 import samr, transport, srvs
 from impacket.dcerpc.v5.dtypes import NULL
 from impacket.smbconnection import *
@@ -36,24 +35,25 @@ except ImportError:
   import readline
 
 class MiniImpacketShell(cmd.Cmd):    
-    def __init__(self):
+    def __init__(self, smbClient):
         cmd.Cmd.__init__(self)
         self.prompt = '# '
-        self.smb = None
+        self.smb = smbClient
+        self.username, self.password, self.domain, self.lmhash, self.nthash, self.aesKey, self.TGT, self.TGS = smbClient.getCredentials()
         self.tid = None
         self.intro = 'Type help for list of commands'
         self.pwd = ''
         self.share = None
-        self.loggedIn = False
-        self.password = None
-        self.lmhash = None
-        self.nthash = None
-        self.username = None
+        self.loggedIn = True
         self.completion = []
 
     def emptyline(self):
         pass
 
+    def precmd(self,line):
+        # switch to unicode
+        return line.decode('utf-8')
+
     def onecmd(self,s):
         retVal = False
         try:
@@ -77,6 +77,7 @@ class MiniImpacketShell(cmd.Cmd):
         print """
  open {host,port=445} - opens a SMB connection against the target host/port
  login {domain/username,passwd} - logs into the current SMB connection, no parameters for NULL connection. If no password specified, it'll be prompted
+ kerberos_login {domain/username,passwd} - logs into the current SMB connection using Kerberos. If no password specified, it'll be prompted. Use the DNS resolvable domain name
  login_hash {domain/username,lmhash:nthash} - logs into the current SMB connection using the password hashes
  logoff - logs off
  shares - list available shares
@@ -103,7 +104,7 @@ class MiniImpacketShell(cmd.Cmd):
             return
         from getpass import getpass
         newPassword = getpass("New Password:")
-        rpctransport = transport.SMBTransport(self.smb.getServerName(), self.smb.getRemoteHost(), filename = r'\samr', smb_connection = self.smb)
+        rpctransport = transport.SMBTransport(self.smb.getRemoteHost(), filename = r'\samr', smb_connection = self.smb)
         dce = rpctransport.get_dce_rpc()
         dce.connect()                     
         dce.bind(samr.MSRPC_UUID_SAMR)
@@ -175,6 +176,40 @@ class MiniImpacketShell(cmd.Cmd):
             logging.info("USER Session Granted")
         self.loggedIn = True
 
+    def do_kerberos_login(self,line):
+        if self.smb is None:
+            logging.error("No connection open")
+            return
+        l = line.split(' ')
+        username = ''
+        password = ''
+        domain = ''
+        if len(l) > 0:
+           username = l[0]
+        if len(l) > 1:
+           password = l[1]
+
+        if username.find('/') > 0:
+           domain, username = username.split('/')
+
+        if domain == '': 
+            logging.error("Domain must be specified for Kerberos login")
+            return
+
+        if password == '' and username != '':
+            from getpass import getpass
+            password = getpass("Password:")
+
+        self.smb.kerberosLogin(username, password, domain=domain)
+        self.password = password
+        self.username = username
+
+        if self.smb.isGuestSession() > 0:
+            logging.info("GUEST Session Granted")
+        else:
+            logging.info("USER Session Granted")
+        self.loggedIn = True
+
     def do_login_hash(self,line): 
         if self.smb is None:
             logging.error("No connection open")
@@ -210,6 +245,7 @@ class MiniImpacketShell(cmd.Cmd):
             logging.error("No connection open")
             return
         self.smb.logoff()
+        del(self.smb)
         self.share = None
         self.smb = None
         self.tid = None
@@ -224,7 +260,7 @@ class MiniImpacketShell(cmd.Cmd):
         if self.loggedIn is False:
             logging.error("Not logged in")
             return
-        rpctransport = transport.SMBTransport(self.smb.getServerName(), self.smb.getRemoteHost(), filename = r'\srvsvc', smb_connection = self.smb)
+        rpctransport = transport.SMBTransport(self.smb.getRemoteHost(), filename = r'\srvsvc', smb_connection = self.smb)
         dce = rpctransport.get_dce_rpc()
         dce.connect()                     
         dce.bind(srvs.MSRPC_UUID_SRVS)
@@ -241,14 +277,14 @@ class MiniImpacketShell(cmd.Cmd):
         if self.loggedIn is False:
             logging.error("Not logged in")
             return
-        rpctransport = transport.SMBTransport(self.smb.getServerName(), self.smb.getRemoteHost(), filename = r'\srvsvc', smb_connection = self.smb)
+        rpctransport = transport.SMBTransport(self.smb.getRemoteHost(), filename = r'\srvsvc', smb_connection = self.smb)
         dce = rpctransport.get_dce_rpc()
         dce.connect()                     
         dce.bind(srvs.MSRPC_UUID_SRVS)
-        resp = srvs.hNetrSessionEnum(dce, NULL, NULL, 502)
+        resp = srvs.hNetrSessionEnum(dce, NULL, NULL, 10)
 
-        for session in resp['InfoStruct']['SessionInfo']['Level502']['Buffer']:
-            print "host: %15s, user: %5s, active: %5d, idle: %5d, type: %5s, transport: %s" % (session['sesi502_cname'][:-1], session['sesi502_username'][:-1], session['sesi502_time'], session['sesi502_idle_time'], session['sesi502_cltype_name'][:-1],session['sesi502_transport'][:-1] )
+        for session in resp['InfoStruct']['SessionInfo']['Level10']['Buffer']:
+            print "host: %15s, user: %5s, active: %5d, idle: %5d" % (session['sesi10_cname'][:-1], session['sesi10_username'][:-1], session['sesi10_time'], session['sesi10_idle_time'])
 
     def do_shares(self, line):
         if self.loggedIn is False:
@@ -287,12 +323,15 @@ class MiniImpacketShell(cmd.Cmd):
             self.smb.closeFile(self.tid,fid)
             self.pwd = oldpwd
             logging.error("Invalid directory")
-        except Exception, e:
+        except SessionError, e:
             if e.getErrorCode() == nt_errors.STATUS_FILE_IS_A_DIRECTORY: 
                pass
             else:
                self.pwd = oldpwd
                raise
+        except Exception, e:
+            self.pwd = oldpwd
+            raise
 
     def do_pwd(self,line):
         if self.loggedIn is False:
@@ -301,6 +340,9 @@ class MiniImpacketShell(cmd.Cmd):
         print self.pwd
 
     def do_ls(self, wildcard, display = True):
+        if self.loggedIn is False:
+            logging.error("Not logged in")
+            return
         if self.tid is None:
             logging.error("No share selected")
             return
@@ -392,33 +434,77 @@ class MiniImpacketShell(cmd.Cmd):
         fh.close()
 
     def do_close(self, line):
-        if self.loggedIn is False:
-            logging.error("Not logged in")
-            return
-        del(self.smb);
+        self.do_logoff(line)
 
 def main():
     print version.BANNER
+    parser = argparse.ArgumentParser(add_help = True, description = "SMB client implementation.")
+
+    parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
+    parser.add_argument('-file', type=argparse.FileType('r'), help='input file with commands to execute in the mini shell')
+    parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
+
+    group = parser.add_argument_group('authentication')
+
+    group.add_argument('-hashes', action="store", metavar = "LMHASH:NTHASH", help='NTLM hashes, format is LMHASH:NTHASH')
+    group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
+    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file (KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the ones specified in the command line')
+    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication (128 or 256 bits)')
 
-    shell = MiniImpacketShell()
     if len(sys.argv)==1:
-        shell.cmdloop()
+        parser.print_help()
+        sys.exit(1)
+
+    options = parser.parse_args()
+
+    if options.debug is True:
+        logging.getLogger().setLevel(logging.DEBUG)
     else:
-        parser = argparse.ArgumentParser()
-        parser.add_argument('-file', type=argparse.FileType('r'), help='input file with commands to execute in the mini shell')
-        options = parser.parse_args()
-        logging.info("Executing commands from %s" % options.file.name)
-        for line in options.file.readlines():
-            if line[0] != '#':
-                print "# %s" % line,
-                shell.onecmd(line)
-            else:
-                print line,
+        logging.getLogger().setLevel(logging.INFO)
+
+    import re
+    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
+
+    if domain is None:
+        domain = ''
+    
+    if password == '' and username != '' and options.hashes is None and options.no_pass is False and options.aesKey is None:
+        from getpass import getpass
+        password = getpass("Password:")
+
+    if options.aesKey is not None:
+        options.k = True
+
+    if options.hashes is not None:
+        lmhash, nthash = options.hashes.split(':')
+    else:
+        lmhash = ''
+        nthash = ''
+ 
+    try: 
+        smbClient = SMBConnection(address, address)
+        if options.k is True:
+            smbClient.kerberosLogin(username, password, domain, lmhash, nthash, options.aesKey )
+        else:
+            smbClient.login(username, password, domain, lmhash, nthash)
+
+        shell = MiniImpacketShell(smbClient)
+
+        if options.file is not None:
+            logging.info("Executing commands from %s" % options.file.name)
+            for line in options.file.readlines():
+                if line[0] != '#':
+                    print "# %s" % line,
+                    shell.onecmd(line)
+                else:
+                    print line,
+        else:
+            shell.cmdloop()
+    except Exception, e:
+        #import traceback
+        #print traceback.print_exc()
+        logging.error(str(e))
 
 if __name__ == "__main__":
-    try:
-        main()
-    except:
-        print "\n"
-        pass
+    main()
 
