@@ -4,8 +4,6 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: samr.py 1244 2014-06-26 13:54:47Z bethus@gmail.com $
-#
 # Author: Alberto Solino
 #
 # Description:
@@ -13,7 +11,7 @@
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB_RPC
+#   at https://github.com/CoreSecurity/impacket/tree/master/impacket/testcases/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
 #   They are located at the end of this file. 
@@ -23,9 +21,10 @@
 from impacket.dcerpc.v5 import ndr
 from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRUNION, NDRPOINTER, NDRUniConformantArray, NDRUniConformantVaryingArray, NDRENUM, NULL
 from impacket.dcerpc.v5.dtypes import *
-from impacket import nt_errors
+from impacket import nt_errors, LOG
 from impacket.uuid import uuidtup_to_bin
 from impacket.dcerpc.v5.enum import Enum
+from impacket.structure import Structure
 
 MSRPC_UUID_SAMR   = uuidtup_to_bin(('12345778-1234-ABCD-EF00-0123456789AC', '1.0'))
 
@@ -1369,6 +1368,136 @@ class PSAM_VALIDATE_OUTPUT_ARG(NDRPOINTER):
         ('Data', SAM_VALIDATE_OUTPUT_ARG),
     )
 
+# 2.2.10 Supplemental Credentials Structures
+
+# 2.2.10.1 USER_PROPERTIES
+class USER_PROPERTIES(Structure):
+    structure = (
+        ('Reserved1','<L=0'),
+        ('Length','<L=0'),
+        ('Reserved2','<H=0'),
+        ('Reserved3','<H=0'),
+        ('Reserved4','96s=""'),
+        ('PropertySignature','<H=0x50'),
+        ('PropertyCount','<H=0'),
+        ('UserProperties',':'),
+    )
+
+# 2.2.10.2 USER_PROPERTY
+class USER_PROPERTY(Structure):
+    structure = (
+        ('NameLength','<H=0'),
+        ('ValueLength','<H=0'),
+        ('Reserved','<H=0'),
+        ('_PropertyName','_-PropertyName', "self['NameLength']"),
+        ('PropertyName',':'),
+        ('_PropertyValue','_-PropertyValue', "self['ValueLength']"),
+        ('PropertyValue',':'),
+    )
+
+# 2.2.10.3 Primary:WDigest - WDIGEST_CREDENTIALS
+class WDIGEST_CREDENTIALS(Structure):
+    structure = (
+        ('Reserved1','B=0'),
+        ('Reserved2','B=0'),
+        ('Version','B=1'),
+        ('NumberOfHashes','B=29'),
+        ('Reserved3','12s=""'),
+        ('Hash1', '16s=""'),
+        ('Hash2', '16s=""'),
+        ('Hash3', '16s=""'),
+        ('Hash4', '16s=""'),
+        ('Hash5', '16s=""'),
+        ('Hash6', '16s=""'),
+        ('Hash7', '16s=""'),
+        ('Hash8', '16s=""'),
+        ('Hash9', '16s=""'),
+        ('Hash10', '16s=""'),
+        ('Hash11', '16s=""'),
+        ('Hash12', '16s=""'),
+        ('Hash13', '16s=""'),
+        ('Hash14', '16s=""'),
+        ('Hash15', '16s=""'),
+        ('Hash16', '16s=""'),
+        ('Hash17', '16s=""'),
+        ('Hash18', '16s=""'),
+        ('Hash19', '16s=""'),
+        ('Hash20', '16s=""'),
+        ('Hash21', '16s=""'),
+        ('Hash22', '16s=""'),
+        ('Hash23', '16s=""'),
+        ('Hash24', '16s=""'),
+        ('Hash25', '16s=""'),
+        ('Hash26', '16s=""'),
+        ('Hash27', '16s=""'),
+        ('Hash28', '16s=""'),
+        ('Hash29', '16s=""'),
+    )
+
+# 2.2.10.5 KERB_KEY_DATA
+class KERB_KEY_DATA(Structure):
+    structure = (
+        ('Reserved1','<H=0'),
+        ('Reserved2','<H=0'),
+        ('Reserved3','<H=0'),
+        ('KeyType','<L=0'),
+        ('KeyLength','<L=0'),
+        ('KeyOffset','<L=0'),
+    )
+
+# 2.2.10.4 Primary:Kerberos - KERB_STORED_CREDENTIAL
+class KERB_STORED_CREDENTIAL(Structure):
+    structure = (
+        ('Revision','<H=3'),
+        ('Flags','<H=0'),
+        ('CredentialCount','<H=0'),
+        ('OldCredentialCount','<H=0'),
+        ('DefaultSaltLength','<H=0'),
+        ('DefaultSaltMaximumLength','<H=0'),
+        ('DefaultSaltOffset','<L=0'),
+        #('Credentials',':'),
+        #('OldCredentials',':'),
+        #('DefaultSalt',':'),
+        #('KeyValues',':'),
+        # All the preceding stuff inside this Buffer
+        ('Buffer',':'),
+    )
+
+# 2.2.10.7 KERB_KEY_DATA_NEW
+class KERB_KEY_DATA_NEW(Structure):
+    structure = (
+        ('Reserved1','<H=0'),
+        ('Reserved2','<H=0'),
+        ('Reserved3','<L=0'),
+        ('IterationCount','<L=0'),
+        ('KeyType','<L=0'),
+        ('KeyLength','<L=0'),
+        ('KeyOffset','<L=0'),
+    )
+
+# 2.2.10.6 Primary:Kerberos-Newer-Keys - KERB_STORED_CREDENTIAL_NEW
+class KERB_STORED_CREDENTIAL_NEW(Structure):
+    structure = (
+        ('Revision','<H=4'),
+        ('Flags','<H=0'),
+        ('CredentialCount','<H=0'),
+        ('ServiceCredentialCount','<H=0'),
+        ('OldCredentialCount','<H=0'),
+        ('OlderCredentialCount','<H=0'),
+        ('DefaultSaltLength','<H=0'),
+        ('DefaultSaltMaximumLength','<H=0'),
+        ('DefaultSaltOffset','<L=0'),
+        ('DefaultIterationCount','<L=0'),
+        #('Credentials',':'),
+        #('ServiceCredentials',':'),
+        #('OldCredentials',':'),
+        #('OlderCredentials',':'),
+        #('DefaultSalt',':'),
+        #('KeyValues',':'),
+        # All the preceding stuff inside this Buffer
+        ('Buffer',':'),
+    )
+
 ################################################################################
 # RPC CALLS
 ################################################################################
@@ -2629,8 +2758,8 @@ def hSamrUnicodeChangePasswordUser2(dce, serverName='\x00', userName='', oldPass
     try:
         from Crypto.Cipher import ARC4
     except Exception:
-        print "Warning: You don't have any crypto installed. You need PyCrypto"
-        print "See http://www.pycrypto.org/"
+        LOG.critical("Warning: You don't have any crypto installed. You need PyCrypto")
+        LOG.critical("See http://www.pycrypto.org/")
     from impacket import crypto, ntlm
 
     if oldPwdHashLM is None and oldPwdHashNT is None:
@@ -2729,7 +2858,6 @@ def hSamrLookupNamesInDomain(dce, domainHandle, names):
         request['Names'].append(entry)
 
     request.fields['Names'].fields['MaximumCount'] = 1000
-    request.dump()
 
     return dce.request(request)
 
@@ -2743,7 +2871,6 @@ def hSamrLookupIdsInDomain(dce, domainHandle, ids):
         request['RelativeIds'].append(entry)
 
     request.fields['RelativeIds'].fields['MaximumCount'] = 1000
-    request.dump()
 
     return dce.request(request)
 
