@@ -5,8 +5,6 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: ese.py 1217 2014-04-16 20:33:05Z bethus@gmail.com $
-#
 # Description:
 #             Microsoft Extensive Storage Engine parser, just focused on trying 
 #             to parse NTDS.dit files (not meant as a full parser, although it might work)
@@ -24,7 +22,7 @@
 # [ ] Parse multi-values properly
 # [ ] Support long values properly
 
-import logging
+from impacket import LOG
 try:
     from collections import OrderedDict
 except:
@@ -161,7 +159,7 @@ CODEPAGE_WESTERN = 1252
 StringCodePages = {
     CODEPAGE_UNICODE : 'utf-16le', 
     CODEPAGE_ASCII   : 'ascii',
-    CODEPAGE_WESTERN : 'ascii',
+    CODEPAGE_WESTERN : 'cp1252',
 }
 
 # Structures
@@ -416,10 +414,10 @@ class ESENT_CATALOG_DATA_DEFINITION_ENTRY(Structure):
         elif dataType == CATALOG_TYPE_LONG_VALUE:
             self.structure += self.other + self.lv_stuff
         elif dataType == CATALOG_TYPE_CALLBACK:
-            logging.error('CallBack types not supported!')
+            LOG.error('CallBack types not supported!')
             raise
         else:
-            logging.error('Unknown ttype 0x%x' % dataType)
+            LOG.error('Unknown catalog type 0x%x' % dataType)
             self.structure = ()
             return Structure.__init__(self,data)
 
@@ -443,6 +441,7 @@ def hexdump(data):
         for j in range(16):
             if i+j < strLen:
                 print "%02X" % ord(x[i+j]),
+
             else:
                 print "  ",
             if j%16 == 7:
@@ -554,7 +553,7 @@ class ESENT_PAGE():
                     #indexEntry.dump()
                 elif self.record['PageFlags'] & FLAGS_LONG_VALUE > 0:
                     # Long Page Value
-                    logging.error('Long value still not supported')
+                    LOG.error('Long value still not supported')
                     raise
                 else:
                     # Table Value
@@ -567,7 +566,7 @@ class ESENT_PAGE():
 
     def getTag(self, tagNum):
         if self.record['FirstAvailablePageTag'] < tagNum:
-            logging.error('Trying to grab an unknown tag 0x%x' % tagNum)
+            LOG.error('Trying to grab an unknown tag 0x%x' % tagNum)
             raise
 
         tags = self.data[-4*self.record['FirstAvailablePageTag']:]
@@ -606,7 +605,7 @@ class ESENT_DB:
         self.mountDB()
 
     def mountDB(self):
-        logging.debug("Mounting DB...")
+        LOG.debug("Mounting DB...")
         if self.__isRemote is True:
             self.__DB = self.__fileName
             self.__DB.open()
@@ -617,9 +616,9 @@ class ESENT_DB:
         self.__pageSize = self.__DBHeader['PageSize']
         self.__DB.seek(0,2)
         self.__totalPages = (self.__DB.tell() / self.__pageSize) -2
-        logging.debug("Database Version:0x%x, Revision:0x%x"% (self.__DBHeader['Version'], self.__DBHeader['FileFormatRevision']))
-        logging.debug("Page Size: %d" % self.__pageSize)
-        logging.debug("Total Pages in file: %d" % self.__totalPages)
+        LOG.debug("Database Version:0x%x, Revision:0x%x"% (self.__DBHeader['Version'], self.__DBHeader['FileFormatRevision']))
+        LOG.debug("Page Size: %d" % self.__pageSize)
+        LOG.debug("Total Pages in file: %d" % self.__totalPages)
         self.parseCatalog(CATALOG_PAGE_NUMBER)
 
     def printCatalog(self):
@@ -662,7 +661,7 @@ class ESENT_DB:
         elif catalogEntry['Type'] == CATALOG_TYPE_LONG_VALUE:
             self.__addLongValue(entry)
         else:
-            logging.error('Unknown type 0x%x' % catalogEntry['Type'])
+            LOG.error('Unknown type 0x%x' % catalogEntry['Type'])
             raise
 
     def __parseItemName(self,entry):
@@ -717,10 +716,10 @@ class ESENT_DB:
 
 
     def readHeader(self):
-        logging.debug("Reading Boot Sector for %s" % self.__volumeName)
+        LOG.debug("Reading Boot Sector for %s" % self.__volumeName)
 
     def getPage(self, pageNum):
-        logging.debug("Trying to fetch page %d (0x%x)" % (pageNum, (pageNum+1)*self.__pageSize))
+        LOG.debug("Trying to fetch page %d (0x%x)" % (pageNum, (pageNum+1)*self.__pageSize))
         self.__DB.seek((pageNum+1)*self.__pageSize, 0)
         data = self.__DB.read(self.__pageSize)
         while len(data) < self.__pageSize:
@@ -932,11 +931,11 @@ class ESENT_DB:
 
                     #print "ID: %d, itemFlag: 0x%x" %( columnRecord['Identifier'], itemFlag)
                     if itemFlag & (TAGGED_DATA_TYPE_COMPRESSED ):
-                        logging.error('Unsupported tag column: %s, flag:0x%x' % (column, itemFlag))
+                        LOG.error('Unsupported tag column: %s, flag:0x%x' % (column, itemFlag))
                         record[column] = None
                     elif itemFlag & TAGGED_DATA_TYPE_MULTI_VALUE:
                         # ToDo: Parse multi-values properly
-                        logging.debug('Multivalue detected in column %s, returning raw results' % (column))
+                        LOG.debug('Multivalue detected in column %s, returning raw results' % (column))
                         record[column] = (tag[offsetItem:][:itemSize].encode('hex'),)
                     else:
                         record[column] = tag[offsetItem:][:itemSize]
@@ -955,7 +954,7 @@ class ESENT_DB:
                 # Let's handle strings
                 if record[column] is not None:
                     if columnRecord['CodePage'] not in StringCodePages:
-                        logging.error('Unknown codepage 0x%x'% columnRecord['CodePage'])
+                        LOG.error('Unknown codepage 0x%x'% columnRecord['CodePage'])
                         raise
                     stringDecoder = StringCodePages[columnRecord['CodePage']]
 
