@@ -4,8 +4,6 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: nmb.py 1224 2014-05-19 14:57:30Z bethus@gmail.com $
-#
 
 
 # -*- mode: python; tab-width: 4 -*-
@@ -854,9 +852,12 @@ class NetBIOSTCPSession(NetBIOSSession):
 
 
     def _setup_connection(self, peer):
-        af, socktype, proto, canonname, sa = socket.getaddrinfo(peer[0], peer[1], 0, socket.SOCK_STREAM)[0]
-        sock = socket.socket(af, socktype, proto)
-        sock.connect(sa)
+        try:
+            af, socktype, proto, canonname, sa = socket.getaddrinfo(peer[0], peer[1], 0, socket.SOCK_STREAM)[0]
+            sock = socket.socket(af, socktype, proto)
+            sock.connect(sa)
+        except Exception, e:
+            raise Exception("%s (%s:%s)" % (e, peer[0], peer[1]))
         return sock
 
     def send_packet(self, data):
